--Call dibs on slots for sounds
freeslot("sfx_fstep0", "sfx_kstep1", "sfx_msjet0", "sfx_msdsh0")

--Decide on the player's feet sounds based on their current skin
addHook("PlayerThink", function(player)
	if not player.mo
		return
	end
	
	-- Assign feet sounds to each of the default characters
	-- feet ["example"] = {
	--							landingsfx = {slow = sfx_ltplop, fast = sfx_hvplop} // fast is optional
	--							walksfx = {sfx_fstep1, sfx_fstep2, sfx_fstep3, ...}
	--							runsfx = {sfx_fstepa, sfx_fstepb, sfx_fstepc, ...}
	--							dashsfx = {sfx_fstepx, sfx_fstepy, sfx_fstepz, ...}
	--							walktiming = function(x) return x==2 or x==6 end, // x is the character's animation frame
	--							runtiming = function(x) return x==1 or x==3 end,
	--							dashtiming = function(x) return x==1 or x==3 end,
	--							airwalk = true // Set if the character's feet don't touch the ground walking
	--							airrun = true // Set if the character's feet don't touch the ground running
	--							stiff = true // Set if the character's walking/running animations only have one frame each
	--						}
	
	local feet = {}
	feet["sonic"] = {
						landingsfx = {slow = nil},
						walksfx = {nil},
						runsfx = {nil},
						dashsfx = {nil},
						walktiming = function(x) return x==2 or x==6 or x==1048578 or x==1048582 end,
						runtiming = function(x) return x==1 or x==3 end,
						dashtiming = function() return leveltime % 11 == 0 end,
					}
	feet["tails"] = {
						landingsfx = {slow = nil, fast = nil},
						walksfx = {nil},
						runsfx = {sfx_s3kba},
						dashsfx = {nil},
						walktiming = function(x) return x==3 or x==7 end,
						runtiming = function() return leveltime % 10 == 0 or leveltime % 10 == 1 end,
						dashtiming = function() return leveltime % 11 == 0 end,
						airrun = true
					}
	feet["knuckles"] = {
						landingsfx = {slow = sfx_s3k4c},
						walksfx = {sfx_kstep1},
						runsfx = {sfx_kstep1},
						dashsfx = {nil},
						walktiming = function(x) return x==2 or x==6 end,
						runtiming = function(x) return x==1 or x==3 end,
						dashtiming = function() return leveltime % 11 == 0 end,
					}
	feet["amy"] = {
						landingsfx = {slow = nil},
						walksfx = {nil},
						runsfx = {nil},
						dashsfx = {nil},
						walktiming = function(x) return x==3 or x==7 end,
						runtiming = function(x) return x==2 or x==6 end,
						dashtiming = function() return leveltime % 11 == 0 end,
					}
	feet["fang"] = {
						landingsfx = {slow = nil},
						walksfx = {nil},
						runsfx = {nil},
						dashsfx = {nil},
						walktiming = function(x) return x==3 or x==7 end,
						runtiming = function(x) return x==1 or x==4 end,
						dashtiming = function() return leveltime % 11 == 0 end,
					}
	feet["metalsonic"] = {
						landingsfx = {slow = sfx_s3k4c},
						walksfx = {sfx_msjet0},
						runsfx = {sfx_msjet0},
						dashsfx = {sfx_msdsh0},
						walktiming = function() return leveltime % 4 == 0 end,
						runtiming = function() return leveltime % 4 == 0 end,
						dashtiming = function() return leveltime % 11 == 0 end,
						airwalk = true,
						airrun = true,
						stiff = true
					}

	-- Pick feet settings for the player based on their skin, default to Sonic's
	player.feet = feet[player.mo.skin] or feet["sonic"]
end)

-- Footstep effect function
local function footSteps(player)
	-- Only play a sound at the start of an animation frame
	if player.mo.frame == player.prevframe
	-- Exception designed for Metal Sonic cause he only has one frame for most of the relevant animations
	and not player.feet["stiff"]
		return
	end
	
	local stepsound = nil
	local stepvolume = 255
	local minvolume = 102
	
	if player.mo.state == S_PLAY_WALK
	and (P_IsObjectOnGround(player.mo) or player.feet["airwalk"])
		-- Check if this is an appropriate frame to play footstep sounds on
		if (player.feet["walktiming"](player.mo.frame))
			-- Randomly pick one of available walking footstep sounds
			stepsound = player.feet["walksfx"][P_RandomRange(1, table.getn(player.feet["walksfx"]))]
		else
			return
		end
		-- Scale footstep volume to how close player is to running animation
		stepvolume = 255 * player.speed / player.runspeed
	elseif (player.mo.state == S_PLAY_RUN
	and (P_IsObjectOnGround(player.mo) or player.feet["airrun"]))
	-- Metal Sonic exception
	or player.mo.state == S_PLAY_FLOAT or player.mo.state == S_PLAY_FLOAT_RUN
		-- Check if this is an appropriate frame to play footstep sounds on
		if (player.feet["runtiming"](player.mo.frame))
			stepsound = player.feet["runsfx"][P_RandomRange(1, table.getn(player.feet["runsfx"]))]
		else
			return
		end
	-- Metal Sonic's so bloody special
	elseif player.mo.state == S_PLAY_DASH
		if (player.feet["dashtiming"](player.mo.frame))
			stepsound = player.feet["dashsfx"][P_RandomRange(1, table.getn(player.feet["dashsfx"]))]
		else
			return
		end
	-- Not walking or running, no footsteps
	else
		return
	end
	
	-- If anything went wrong, default to default footstep at 255
	S_StartSoundAtVolume(player.mo, stepsound or sfx_fstep0, max(stepvolume, minvolume) or 255)
end

-- Landing thud effect function
local function landingThud(player)
	local thudsound = nil
	-- Maximum landing thud volume is reached when you hit the ground at a speed of thudmax * FRACUNIT
	local thudmax = 10
	-- Scale landing thud sound to how fast they hit the ground
	local thudvolume = 255 * min(abs(player.prevmomz or 0), thudmax*FRACUNIT) / thudmax / FRACUNIT
	
	-- Player is moving on foot slowly or not at all
	if player.mo.state == S_PLAY_STND or player.mo.state == S_PLAY_WALK
		-- Play the slow landing thud
		thudsound = player.feet["landingsfx"]["slow"]
	-- Player is, or was, moving on foot quickly
	elseif player.mo.state == S_PLAY_RUN or player.mo.state == S_PLAY_SKID or player.mo.state == S_PLAY_DASH
		-- Default to slow landing thud if there isn't a fast thud
		thudsound = player.feet["landingsfx"]["fast"] or player.feet["landingsfx"]["slow"]
	-- Player is doing something else so their feet shouldn't be hitting the ground
	else
		return
	end
	
	S_StartSoundAtVolume(player.mo, thudsound or sfx_kc50, thudvolume or 255)
end

addHook("ThinkFrame", function()
	for player in players.iterate()
		-- If spectator, get outta here
		if not (player.mo and player.mo.valid)
			continue
		end
		
		-- Gotta be on the ground for your feet to make noises
		if P_IsObjectOnGround(player.mo) and player.prevground == false
			-- Landing thud effects
			landingThud(player)
		else
			-- Footstep effects
			footSteps(player)
		end
		
		--Set up vars for next frame's checks
		player.prevframe = player.mo.frame
		player.prevground = P_IsObjectOnGround(player.mo)
		player.prevmomz = player.mo.momz
	end
end, MT_PLAYER)
